:: GsplitWithVSSCustom.bat - This script will split a large file into chunks using Volume Shadowing and portable Gsplit
:: Usage: GsplitWithVSS.bat "<inputfilespec>" "<outputFolder>" "<OS-type>" Note the output folder should NOT have a trailing /
:: Valid OS-types are: XP, 2003, 32 and 64
:: There is a package of file dependencies that accompany this script including gsplit and vshadow distributions
:: 12-Feb-2010 - Portions adapted from tech posting on the Internet - MDR - Dr.Backup
:: 03/21/2011 - This module is customized to backup .edb and .stm files for Exchange files. User specifies the .edb only - MDR
:: 21-May-2012 - Write out sentinel file if Gsplit or VSS fail - MDR - Dr.Backup
:: 10-21-2013 - Minor changes to clean up partially split files in gsplit exits with error code other than success.



SETLOCAL

:: Customization parameters which can be adjusted
SET MIN_FILE_SIZE="300000000"
SET FREE_DRIVE_LETTER=Q:
SET CHUNK_SIZE="250"
SET CHUNK_UNITS="MB"

:: Do not change anything below this line


:: The version of VSHADOW is different on each OS. Specify it as parameter 3
:: Valid values are XP, 2003, 32 or 64 - default in build will be 32 (Vista 32 or Windows 7 -32bit)
SET OS=%3

:: If this is a callback from VSHADOW, then skip parameter setup
IF NOT "%CALLBACK_SCRIPT%"=="" GOTO :IS_CALLBACK

:: Extract the drive letter for use for the VSS Shadow, the relative path to append to the FREE_DRIVE_LETTER
:: and the destination folder to put the split files into. These parameters are all coming from the command line
SET SOURCE_DRIVE_LETTER=%~d1
SET SOURCE_RELATIVE_PATH=%~pnx1
SET DESTINATION_PATH=%2
SET FILE_TO_COPY=%~nx1

:: We will be telling VSHADOW to do a recursive callback to this batch job, specified as parameter %0 in command line
SET CALLBACK_SCRIPT=%~dpnx0

:: Write error file here. If VSS snapshot succeeds, this will be deleted.
ECHO VSSGSPLITERROR-%FILE_TO_COPY%(VSSError) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITERROR##.TXT"

:: Tell VSHADOW where to write the Volume shadow filespec after created
:: We will execute this batch script after the callback - Hardcoded the script name
SET TEMP_GENERATED_SCRIPT=GeneratedVarsTempScript.cmd

:: Lets actually create the volume shadow set now. VSHADOW will call us back when done and load definitions
:: into the TEMP_GENERATED_SCRIPT command file.
"%~dp0VSHADOW%OS%.EXE" -script="%TEMP_GENERATED_SCRIPT%" -exec="%CALLBACK_SCRIPT%" %SOURCE_DRIVE_LETTER%

:: By time we get here, the callback has occured and we are done with the temporarily generated script to give us
:: the shadow file spec. Get rid of temp file
DEL /F %TEMP_GENERATED_SCRIPT%

:: We are done.
GOTO :EOF

:: Here is the magic. We are called and wind up here while the Shadow volume still exists and is mapped to a drive
:: Lets do our testing and Gsplit if file big enough

:IS_CALLBACK
SETLOCAL

:: Lets call the script created by VSHADOW to let us know the device name of the shadow volume so we can map it to a drive
:: The script should set the SHADOW_DEVICE_1 env variable
CALL %TEMP_GENERATED_SCRIPT%

:: To use the shadow volume freely, lets assign it a drive letter
:: We are using the DOSDEV utility here provided in the package
:: 8/31/2011 - remove the drive before attempting to assign
"%~dp0DOSDEV" -d -r %FREE_DRIVE_LETTER%
"%~dp0DOSDEV" %FREE_DRIVE_LETTER% %SHADOW_DEVICE_1%

:: Process the main file here
:: Little housekeeping here to get rid of any previously generated gsplit files in this folder
:: We had to remove the quote marks from the original path argument if they were present
DEL /Q "%DESTINATION_PATH:"=%\GS-*.*"

:: If the file itself is not big enough, it cannot be split. Let's test first.
:: Reason we test is the gsplit will hang if fed too small a file :-(
:: Use a small shell script to do the file size testing. Included in package

START /w cscript "%~dp0Size.vbs" "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%" ">" %MIN_FILE_SIZE%
IF NOT ERRORLEVEL 1 GOTO RUNGSPLIT

:: We are here because the file is too small to split. Let's just copy the file anyway
:: 12/29/2011 - added touch code since copy doesn't update MDT
COPY /Y "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%" "%DESTINATION_PATH:"=%\GS-%FILE_TO_COPY%"
cd /d "%DESTINATION_PATH:"=%"
COPY /B "%DESTINATION_PATH:"=%\GS-%FILE_TO_COPY%" +,,
GOTO FILE2

:: Well we really do have a big file to split. Let's break it up into 250MB chunks
:RUNGSPLIT

:: Write out the gsplit.gsf configuration file that tells portable gsplit what to do
Echo [MAIN]> "%~dp0gsplit.gsf"
Echo Source=%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%>> "%~dp0gsplit.gsf"
Echo DestFolder=%DESTINATION_PATH:"=%>> "%~dp0gsplit.gsf"
Echo PieSize=%CHUNK_SIZE:"=% >> "%~dp0gsplit.gsf"
Echo PieSizeUnit=%CHUNK_UNITS:"=% >> "%~dp0gsplit.gsf"
Echo PieMask=GS-{ofw}.{oru} >> "%~dp0gsplit.gsf"
Echo SplitMethod=0 >> "%~dp0gsplit.gsf"
Echo TypePieSpanned=0 >> "%~dp0gsplit.gsf"
Echo Title=Gsplit-Backup >> "%~dp0gsplit.gsf"

:: Now that everything is setup, we can use the portable gsplit utility
"%~dp0gsplit.exe" "no" "-b-i-t" "%~dp0gsplit.gsf"

:: Test for gsplit error here and write out sentinel file if needed
IF %ERRORLEVEL% EQU 0 GOTO FILE2
DEL /Q "%DESTINATION_PATH:"=%\GS-*.*"
DEL /Q "%DESTINATION_PATH:"=%STM\GS-*.*"
ECHO VSSGSPLITERROR-%FILE_TO_COPY%(GsplitError) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITERROR##.TXT"
GOTO :EOF

:FILE2
:: Automatically grab the matching .stm and process it same way as original
:: Little housekeeping here to get rid of any previously generated gsplit files in this folder
:: We had to remove the quote marks from the original path argument if they were present
DEL /Q "%DESTINATION_PATH:"=%STM\GS-*.*"

:: If the file itself is not big enough, it cannot be split. Let's test first.
:: Reason we test is the gsplit will hang if fed too small a file :-(
:: Use a small shell script to do the file size testing. Included in package

START /w cscript "%~dp0Size.vbs" "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH:.edb=.stm%" ">" %MIN_FILE_SIZE%
IF NOT ERRORLEVEL 1 GOTO RUNGSPLIT2

:: We are here because the file is too small to split. Let's just copy the file anyway
:: 12/29/2011 - add touch code here since copy doesn't update MDT
COPY /Y "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH:.edb=.stm%" "%DESTINATION_PATH:"=%STM\GS-%FILE_TO_COPY:.edb=.stm%"
cd /d "%DESTINATION_PATH:"=%STM"
COPY /B "%DESTINATION_PATH:"=%STM\GS-%FILE_TO_COPY:.edb=.stm%" +,,

GOTO :EOF

:: Well we really do have a big file to split. Let's break it up into 250MB chunks
:RUNGSPLIT2

:: Write out the gsplit.gsf configuration file that tells portable gsplit what to do
Echo [MAIN]> "%~dp0gsplit.gsf"
Echo Source=%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH:.edb=.stm%>> "%~dp0gsplit.gsf"
Echo DestFolder=%DESTINATION_PATH:"=%STM>> "%~dp0gsplit.gsf"
Echo PieSize=%CHUNK_SIZE:"=% >> "%~dp0gsplit.gsf"
Echo PieSizeUnit=%CHUNK_UNITS:"=% >> "%~dp0gsplit.gsf"
Echo PieMask=GS-{ofw}.{oru} >> "%~dp0gsplit.gsf"
Echo SplitMethod=0 >> "%~dp0gsplit.gsf"
Echo TypePieSpanned=0 >> "%~dp0gsplit.gsf"
Echo Title=Gsplit-Backup >> "%~dp0gsplit.gsf"

:: Now that everything is setup, we can use the portable gsplit utility
"%~dp0gsplit.exe" "no" "-b-i-t" "%~dp0gsplit.gsf"

:: Test for gsplit error here and write out sentinel file if needed
IF %ERRORLEVEL% EQU 0 GOTO :EOF 
DEL /Q "%DESTINATION_PATH:"=%\GS-*.*"
DEL /Q "%DESTINATION_PATH:"=%STM\GS-*.*"
ECHO VSSGSPLITERROR-%FILE_TO_COPY%(GsplitError) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITERROR##.TXT"

:EOF
:: All done here